@@ -1,5 +1,5 @@
 /* ldapdelete.c - simple program to delete an entry using LDAP */
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapdelete.c,v 1.80.2.5 2002/08/30 04:31:10 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapdelete.c,v 1.80.2.6 2002/12/02 18:20:19 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -197,7 +197,7 @@ main( int argc, char **argv )
 
 		if ( strcasecmp( control, "manageDSAit" ) == 0 ) {
 			if( cvalue != NULL ) {
-				fprintf( stderr, "manageDSAit: no control value expected" );
+				fprintf( stderr, "manageDSAit: no control value expected\n" );
 				usage(prog);
 				return EXIT_FAILURE;
 			}
@@ -208,7 +208,7 @@ main( int argc, char **argv )
 			
 		} else if ( strcasecmp( control, "noop" ) == 0 ) {
 			if( cvalue != NULL ) {
-				fprintf( stderr, "noop: no control value expected" );
+				fprintf( stderr, "noop: no control value expected\n" );
 				usage(prog);
 				return EXIT_FAILURE;
 			}
