@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.111.2.12 2002/09/06 03:16:38 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.111.2.15 2002/12/02 18:20:19 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -283,7 +283,7 @@ main( int argc, char **argv )
 		if ( strcasecmp( control, "mv" ) == 0 ) {
 			/* ValuesReturnFilter control */
 			if (valuesReturnFilter!=0) {
-				fprintf( stderr, "ValuesReturnFilter previously specified");
+				fprintf( stderr, "ValuesReturnFilter previously specified\n" );
 				return EXIT_FAILURE;
 			}
 			valuesReturnFilter= 1 + crit;
@@ -393,7 +393,7 @@ main( int argc, char **argv )
 
 		if ( strcasecmp( control, "manageDSAit" ) == 0 ) {
 			if( cvalue != NULL ) {
-				fprintf( stderr, "manageDSAit: no control value expected" );
+				fprintf( stderr, "manageDSAit: no control value expected\n" );
 				usage(prog);
 				return EXIT_FAILURE;
 			}
@@ -404,7 +404,7 @@ main( int argc, char **argv )
 			
 		} else if ( strcasecmp( control, "noop" ) == 0 ) {
 			if( cvalue != NULL ) {
-				fprintf( stderr, "noop: no control value expected" );
+				fprintf( stderr, "noop: no control value expected\n" );
 				usage(prog);
 				return EXIT_FAILURE;
 			}
@@ -1030,8 +1030,14 @@ main( int argc, char **argv )
 	}
 
 	if (ldif < 2 ) {
-		printf( "#\n# LDAPv%d\n# filter%s: %s\n# requesting: ",
+		printf( "#\n"
+			"# LDAPv%d\n"
+			"# base <%s> with scope %s\n"
+			"# filter%s: %s\n"
+			"# requesting: ",
 			version,
+			base, (scope == LDAP_SCOPE_BASE) ? "base"
+				: ((scope == LDAP_SCOPE_ONELEVEL) ? "one" : "sub"),
 			infile != NULL ? " pattern" : "",
 			filtpattern );
 
@@ -1494,13 +1500,17 @@ static void print_extended(
 		exit( EXIT_FAILURE );
 	}
 
-	write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_VALUE,
-		"extended", retoid, retoid ? strlen(retoid) : 0 );
+	if ( ldif < 2 ) {
+		write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_VALUE,
+			"extended", retoid, retoid ? strlen(retoid) : 0 );
+	}
 	ber_memfree( retoid );
 
 	if(retdata) {
-		write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_BINARY,
-			"data", retdata->bv_val, retdata->bv_len );
+		if ( ldif < 2 ) {
+			write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_BINARY,
+				"data", retdata->bv_val, retdata->bv_len );
+		}
 		ber_bvfree( retdata );
 	}
 
@@ -1528,15 +1538,18 @@ static void print_partial(
 		exit( EXIT_FAILURE );
 	}
 
-	write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_VALUE,
-		"partial", retoid, retoid ? strlen(retoid) : 0 );
+	if ( ldif < 2 ) {
+		write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_VALUE,
+			"partial", retoid, retoid ? strlen(retoid) : 0 );
+	}
 
 	ber_memfree( retoid );
 
 	if( retdata ) {
-		write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_BINARY,
-			"data", 
-			retdata->bv_val, retdata->bv_len );
+		if ( ldif < 2 ) {
+			write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_BINARY,
+				"data", retdata->bv_val, retdata->bv_len );
+		}
 
 		ber_bvfree( retdata );
 	}
@@ -1635,8 +1648,9 @@ static void print_ctrls(
 		struct berval *b64 = NULL;
 		ber_len_t len;
 		char *str;
-			
-		len = strlen( ctrls[i]->ldctl_oid );
+
+		len = ldif ? 2 : 0;
+		len += strlen( ctrls[i]->ldctl_oid );
 
 		/* add enough for space after OID and the critical value itself */
 		len += ctrls[i]->ldctl_iscritical
@@ -1660,7 +1674,12 @@ static void print_ctrls(
 		}
 
 		str = malloc( len + 1 );
-		strcpy( str, ctrls[i]->ldctl_oid );
+		if ( ldif ) {
+			strcpy( str, ": " );
+		} else {
+			str[0] = '\0';
+		}
+		strcat( str, ctrls[i]->ldctl_oid );
 		strcat( str, ctrls[i]->ldctl_iscritical
 			? " true" : " false" );
 
@@ -1669,8 +1688,10 @@ static void print_ctrls(
 			strcat(str, b64->bv_val );
 		}
 
-		write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_VALUE,
-			"control", str, len );
+		if ( ldif < 2 ) {
+			write_ldif( ldif ? LDIF_PUT_COMMENT : LDIF_PUT_VALUE,
+				"control", str, len );
+		}
 
 		free( str );
 		ber_bvfree( b64 );
