@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/util-int.c,v 1.17.8.8 2002/01/04 20:38:22 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/util-int.c,v 1.33.2.1 2002/12/03 21:58:21 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -59,9 +59,6 @@
 # ifndef USE_CTIME_R
 	static ldap_pvt_thread_mutex_t ldap_int_ctime_mutex;
 # endif
-# if !defined( HAVE_GETHOSTBYNAME_R ) || !defined( HAVE_GETHOSTBYADDR_R )
-	static ldap_pvt_thread_mutex_t ldap_int_gethostby_mutex;
-# endif
 # ifdef HAVE_RES_QUERY
 	ldap_pvt_thread_mutex_t ldap_int_resolv_mutex;
 # endif
@@ -92,8 +89,8 @@ char *ldap_pvt_ctime( const time_t *tp, char *buf )
 #endif	
 }
 
-#define BUFSTART 1024
-#define BUFMAX (32*1024)
+#define BUFSTART (1024-32)
+#define BUFMAX (32*1024-32)
 
 static char *safe_realloc( char **buf, int len );
 
@@ -148,7 +145,7 @@ int ldap_pvt_gethostbyname_a(
 	int	retval;
 	*buf = NULL;
 	
-	ldap_pvt_thread_mutex_lock( &ldap_int_gethostby_mutex );
+	ldap_pvt_thread_mutex_lock( &ldap_int_resolv_mutex );
 	
 	he = gethostbyname( name );
 	
@@ -163,7 +160,7 @@ int ldap_pvt_gethostbyname_a(
 		retval = 0;
 	}
 	
-	ldap_pvt_thread_mutex_unlock( &ldap_int_gethostby_mutex );
+	ldap_pvt_thread_mutex_unlock( &ldap_int_resolv_mutex );
 	
 	return retval;
 #else	
@@ -229,7 +226,7 @@ int ldap_pvt_gethostbyaddr_a(
 	int	retval;
 	*buf = NULL;   
 	
-	ldap_pvt_thread_mutex_lock( &ldap_int_gethostby_mutex );
+	ldap_pvt_thread_mutex_lock( &ldap_int_resolv_mutex );
 	
 	he = gethostbyaddr( addr, len, type );
 	
@@ -244,7 +241,7 @@ int ldap_pvt_gethostbyaddr_a(
 		retval = 0;
 	}
 	
-	ldap_pvt_thread_mutex_unlock( &ldap_int_gethostby_mutex );
+	ldap_pvt_thread_mutex_unlock( &ldap_int_resolv_mutex );
 	
 	return retval;   
 #else /* gethostbyaddr() */
@@ -274,10 +271,6 @@ void ldap_int_utils_init( void )
 	ldap_pvt_thread_mutex_init( &ldap_int_ctime_mutex );
 #endif
 
-#if !defined( HAVE_GETHOSTBYNAME_R ) || !defined( HAVE_GETHOSTBYADDR_R )
-	ldap_pvt_thread_mutex_init( &ldap_int_gethostby_mutex );
-#endif
-
 #ifdef HAVE_RES_QUERY
 	ldap_pvt_thread_mutex_init( &ldap_int_resolv_mutex );
 #endif
