@@ -1,5 +1,5 @@
 /* back-bdb.h - bdb back-end header file */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/back-bdb.h,v 1.54.2.6 2002/09/19 20:04:16 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/back-bdb.h,v 1.54.2.9 2002/11/28 22:22:36 kurt Exp $ */
 /*
  * Copyright 2000-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -9,9 +9,8 @@
 #define _BACK_BDB_H_
 
 #include <portable.h>
-#include <db.h>
-
 #include "slap.h"
+#include <db.h>
 
 LDAP_BEGIN_DECL
 
@@ -114,6 +113,9 @@ struct bdb_info {
 
 	ID			bi_lastid;
 	ldap_pvt_thread_mutex_t	bi_lastid_mutex;
+#ifdef LDAP_CLIENT_UPDATE
+	LDAP_LIST_HEAD(pl, slap_op) psearch_list;
+#endif
 };
 
 #define bi_id2entry	bi_databases[BDB_ID2ENTRY]
