@@ -1,5 +1,5 @@
 /* init.c - initialize bdb backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/init.c,v 1.75.2.12 2002/09/19 20:04:17 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/init.c,v 1.75.2.14 2002/11/11 02:42:38 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -37,13 +37,13 @@ struct berval bdb_uuid = { 0, NULL };
 typedef void * db_malloc(size_t);
 typedef void * db_realloc(void *, size_t);
 
+#if 0
 static int
 bdb_open( BackendInfo *bi )
 {
 	return 0;
 }
 
-#if 0
 static int
 bdb_destroy( BackendInfo *bi )
 {
@@ -93,6 +93,10 @@ bdb_db_init( BackendDB *be )
 
 	bdb->bi_lock_detect = DB_LOCK_DEFAULT;
 
+#ifdef LDAP_CLIENT_UPDATE
+	LDAP_LIST_INIT (&bdb->psearch_list);
+#endif
+
 	ldap_pvt_thread_mutex_init( &bdb->bi_database_mutex );
 	ldap_pvt_thread_mutex_init( &bdb->bi_lastid_mutex );
 	ldap_pvt_thread_mutex_init( &bdb->bi_cache.lru_mutex );
@@ -512,7 +516,7 @@ bdb_initialize(
 #ifdef NEW_LOGGING
 	LDAP_LOG( BACK_BDB, ENTRY, "bdb_db_initialize\n", 0, 0, 0 );
 #else
-	Debug( LDAP_DEBUG_TRACE, "bdb_open: initialize BDB backend\n",
+	Debug( LDAP_DEBUG_TRACE, "bdb_initialize: initialize BDB backend\n",
 		0, 0, 0 );
 #endif
 
@@ -541,7 +545,7 @@ bdb_initialize(
 				"\texpected: %s \tgot: %s\n", DB_VERSION_STRING, version, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
-				"bdb_open: version mismatch\n"
+				"bdb_initialize: version mismatch\n"
 				"\texpected: " DB_VERSION_STRING "\n"
 				"\tgot: %s \n", version, 0, 0 );
 #endif
@@ -549,9 +553,9 @@ bdb_initialize(
 
 #ifdef NEW_LOGGING
 		LDAP_LOG( BACK_BDB, DETAIL1, 
-			"bdb_db_initialize: bdb_open: %s\n", version, 0, 0 );
+			"bdb_db_initialize: %s\n", version, 0, 0 );
 #else
-		Debug( LDAP_DEBUG_ANY, "bdb_open: %s\n",
+		Debug( LDAP_DEBUG_ANY, "bdb_initialize: %s\n",
 			version, 0, 0 );
 #endif
 	}
@@ -611,6 +615,9 @@ bdb_initialize(
 
 	bi->bi_chk_referrals = bdb_referrals;
 	bi->bi_operational = bdb_operational;
+#ifdef SLAP_X_FILTER_HASSUBORDINATES
+	bi->bi_has_subordinates = bdb_hasSubordinates;
+#endif /* SLAP_X_FILTER_HASSUBORDINATES */
 	bi->bi_entry_release_rw = bdb_entry_release;
 
 	/*
