@@ -1,5 +1,5 @@
 /* search.c - search operation */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.54.2.12 2002/09/17 02:15:08 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.54.2.15 2002/11/26 17:37:09 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -67,10 +67,6 @@ bdb_search(
 	struct slap_limits_set *limit = NULL;
 	int isroot = 0;
 
-#ifdef SLAP_X_FILTER_HASSUBORDINATES
-	int		filter_hasSubordinates = 0;
-#endif /* SLAP_X_FILTER_HASSUBORDINATES */
-
 	u_int32_t	locker;
 	DB_LOCK		lock;
 
@@ -81,6 +77,15 @@ bdb_search(
 		0, 0, 0);
 #endif
 
+#ifdef LDAP_CLIENT_UPDATE
+	if ( op->o_clientupdate_type & SLAP_LCUP_PERSIST ) {
+		bdb_add_psearch_spec( be, conn, op, base, base, scope,
+			deref, slimit, tlimit, filter, filterstr, attrs, attrsonly );
+		return LDAP_SUCCESS;
+	}
+#endif
+
+
 	manageDSAit = get_manageDSAit( op );
 
 	rc = LOCK_ID (bdb->bi_dbenv, &locker );
@@ -242,13 +247,15 @@ dn2entry_retry:
 		/* if requested limit higher than hard limit, abort */
 		} else if ( tlimit > limit->lms_t_hard ) {
 			/* no hard limit means use soft instead */
-			if ( limit->lms_t_hard == 0 && tlimit > limit->lms_t_soft ) {
+			if ( limit->lms_t_hard == 0
+					&& limit->lms_t_soft > -1
+					&& tlimit > limit->lms_t_soft ) {
 				tlimit = limit->lms_t_soft;
 
 			/* positive hard limit means abort */
 			} else if ( limit->lms_t_hard > 0 ) {
 				send_search_result( conn, op, 
-						LDAP_UNWILLING_TO_PERFORM,
+						LDAP_ADMINLIMIT_EXCEEDED,
 						NULL, NULL, NULL, NULL, 0 );
 				rc = 0;
 				goto done;
@@ -264,13 +271,15 @@ dn2entry_retry:
 		/* if requested limit higher than hard limit, abort */
 		} else if ( slimit > limit->lms_s_hard ) {
 			/* no hard limit means use soft instead */
-			if ( limit->lms_s_hard == 0 && slimit > limit->lms_s_soft ) {
+			if ( limit->lms_s_hard == 0
+					&& limit->lms_s_soft > -1
+					&& slimit > limit->lms_s_soft ) {
 				slimit = limit->lms_s_soft;
 
 			/* positive hard limit means abort */
 			} else if ( limit->lms_s_hard > 0 ) {
 				send_search_result( conn, op, 
-						LDAP_UNWILLING_TO_PERFORM,
+						LDAP_ADMINLIMIT_EXCEEDED,
 						NULL, NULL, NULL, NULL, 0 );
 				rc = 0;	
 				goto done;
@@ -333,14 +342,6 @@ dn2entry_retry:
 		}
 	}
 
-#ifdef SLAP_X_FILTER_HASSUBORDINATES
-	/*
-	 * is hasSubordinates used in the filter ?
-	 * FIXME: we may compute this directly when parsing the filter
-	 */
-	filter_hasSubordinates = filter_has_subordinates( filter );
-#endif /* SLAP_X_FILTER_HASSUBORDINATES */
-
 #ifdef LDAP_CLIENT_UPDATE
 	if ( op->o_clientupdate_type & SLAP_LCUP_SYNC ) {
 		lcupf.f_choice = LDAP_FILTER_AND;
@@ -373,9 +374,6 @@ dn2entry_retry:
 		id = bdb_idl_next( candidates, &cursor ) )
 	{
 		int		scopeok = 0;
-#ifdef SLAP_X_FILTER_HASSUBORDINATES
-		Attribute	*hasSubordinates = NULL;
-#endif /* SLAP_X_FILTER_HASSUBORDINATES */
 
 		/* check for abandon */
 		if ( op->o_abandon ) {
@@ -539,31 +537,6 @@ id2entry_retry:
 			goto loop_continue;
 		}
 
-#ifdef SLAP_X_FILTER_HASSUBORDINATES
-		/*
-		 * if hasSubordinates is used in the filter,
-		 * append it to the entry's attributes
-		 */
-		if ( filter_hasSubordinates ) {
-			int	hs;
-
-			rc = bdb_hasSubordinates( be, conn, op, e, &hs);
-			if ( rc != LDAP_SUCCESS ) {
-				goto loop_continue;
-			}
-
-			hasSubordinates = slap_operational_hasSubordinate(
-				hs == LDAP_COMPARE_TRUE );
-
-			if ( hasSubordinates == NULL ) {
-				goto loop_continue;
-			}
-
-			hasSubordinates->a_next = e->e_attrs;
-			e->e_attrs = hasSubordinates;
-		}
-#endif /* SLAP_X_FILTER_HASSUBORDINATES */
-
 		/* if it matches the filter and scope, send it */
 #ifdef LDAP_CLIENT_UPDATE
 		if ( op->o_clientupdate_type & SLAP_LCUP_SYNC ) {
@@ -574,21 +547,6 @@ id2entry_retry:
 			rc = test_filter( be, conn, op, e, filter );
 		}
 
-#ifdef SLAP_X_FILTER_HASSUBORDINATES
-		if ( hasSubordinates ) {
-			/*
-			 * FIXME: this is fairly inefficient, because 
-			 * if hasSubordinates is among the required
-			 * attrs, it will be added again later;
-			 * maybe we should leave it and check
-			 * check later if it's already present,
-			 * if required
-			 */
-			e->e_attrs = e->e_attrs->a_next;
-			attr_free( hasSubordinates );
-		}
-#endif /* SLAP_X_FILTER_HASSUBORDINATES */
-
 		if ( rc == LDAP_COMPARE_TRUE ) {
 			struct berval	dn;
 
@@ -866,7 +824,7 @@ done:
 		bdb_cache_return_entry_r ( bdb->bi_dbenv, &bdb->bi_cache, e, &lock );
 	}
 
-#ifdef LDAP_CLIENT_UDATE
+#ifdef LDAP_CLIENT_UPDATE
 	if ( op->o_clientupdate_type & SLAP_LCUP_SYNC ) {
 		if ( csnfeq.f_ava != NULL && csnfeq.f_av_value.bv_val != NULL ) {
 			ch_free( csnfeq.f_av_value.bv_val );
@@ -987,7 +945,7 @@ static int search_candidates(
 	 */
 	if (!oc_filter(filter, 1, &depth) && !get_subentries_visibility(op) ) {
 		if( !get_manageDSAit(op) ) { /* match referrals */
-			struct berval bv_ref = { sizeof("REFERRAL")-1, "REFERRAL" };
+			struct berval bv_ref = { sizeof("referral")-1, "referral" };
 			rf.f_choice = LDAP_FILTER_EQUALITY;
 			rf.f_ava = &aa_ref;
 			rf.f_av_desc = slap_schema.si_ad_objectClass;
@@ -998,7 +956,7 @@ static int search_candidates(
 
 #ifdef BDB_ALIASES
 		if( deref & LDAP_DEREF_SEARCHING ) { /* match aliases */
-			struct berval bv_alias = { sizeof("ALIAS")-1, "ALIAS" };
+			struct berval bv_alias = { sizeof("alias")-1, "alias" };
 			af.f_choice = LDAP_FILTER_EQUALITY;
 			af.f_ava = &aa_alias;
 			af.f_av_desc = slap_schema.si_ad_objectClass;
