@@ -122,12 +122,12 @@ meta_back_modify(
 		
 	case REWRITE_REGEXEC_UNWILLING:
 		send_ldap_result( conn, op, LDAP_UNWILLING_TO_PERFORM,
-				NULL, NULL, NULL, NULL );
+				NULL, "Operation not allowed", NULL, NULL );
 		return -1;
 
 	case REWRITE_REGEXEC_ERR:
 		send_ldap_result( conn, op, LDAP_OTHER,
-				NULL, NULL, NULL, NULL );
+				NULL, "Rewrite error", NULL, NULL );
 		return -1;
 	}
 
@@ -179,12 +179,18 @@ meta_back_modify(
 				ml->sml_bvalues, conn );
 		}
 
-		for (j = 0; ml->sml_bvalues[ j ].bv_val; j++);
-		mods[ i ].mod_bvalues = (struct berval **)ch_malloc((j+1) *
-			sizeof(struct berval *));
-		for (j = 0; ml->sml_bvalues[ j ].bv_val; j++)
-			mods[ i ].mod_bvalues[ j ] = &ml->sml_bvalues[j];
-		mods[ i ].mod_bvalues[ j ] = NULL;
+		if ( ml->sml_bvalues != NULL ){
+			for (j = 0; ml->sml_bvalues[ j ].bv_val; j++);
+			mods[ i ].mod_bvalues = (struct berval **)ch_malloc((j+1) *
+				sizeof(struct berval *));
+			for (j = 0; ml->sml_bvalues[ j ].bv_val; j++)
+				mods[ i ].mod_bvalues[ j ] = &ml->sml_bvalues[j];
+			mods[ i ].mod_bvalues[ j ] = NULL;
+
+		} else {
+			mods[ i ].mod_bvalues = NULL;
+		}
+
 		i++;
 	}
 	modv[ i ] = 0;
