@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/controls.c,v 1.30.2.7 2002/09/17 02:26:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/controls.c,v 1.30.2.8 2002/10/29 01:26:20 kurt Exp $ */
 /* 
  * Copyright 1999-2002 The OpenLDAP Foundation.
  * All rights reserved.
@@ -318,6 +318,9 @@ int get_ctrls(
 			case LDAP_REQ_UNBIND:
 				tagmask = SLAP_CTRL_UNBIND;
 				break;
+			case LDAP_REQ_ABANDON:
+				tagmask = SLAP_CTRL_ABANDON;
+				break;
 			case LDAP_REQ_EXTENDED:
 				/* FIXME: check list of extended operations */
 				tagmask = ~0U;
