@@ -1,5 +1,5 @@
 /* oc.c - object class routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/oc.c,v 1.34.2.5 2002/10/10 01:41:41 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/oc.c,v 1.34.2.7 2002/11/26 18:26:19 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -67,7 +67,7 @@ int is_entry_objectclass(
 	}
 
 	if( set_flags && ( e->e_ocflags & SLAP_OC__END )) {
-		return (e->e_ocflags & oc->soc_flags) ? 1 : 0;
+		return (e->e_ocflags & oc->soc_flags) == oc->soc_flags;
 	}
 
 	/*
@@ -105,7 +105,7 @@ int is_entry_objectclass(
 	}
 	e->e_ocflags |= SLAP_OC__END;	/* We've finished this */
 
-	return (e->e_ocflags & oc->soc_flags);
+	return (e->e_ocflags & oc->soc_flags) == oc->soc_flags;
 }
 
 
@@ -491,7 +491,8 @@ oc_schema_info( Entry *e )
 		Debug( LDAP_DEBUG_TRACE, "Merging oc [%ld] %s\n",
 	       (long) vals[0].bv_len, vals[0].bv_val, 0 );
 #endif
-		attr_merge( e, ad_objectClasses, vals );
+		if( attr_merge( e, ad_objectClasses, vals ) )
+			return -1;
 		ldap_memfree( vals[0].bv_val );
 	}
 	return 0;
