@@ -1,5 +1,5 @@
 /* syntax.c - routines to manage syntax definitions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/syntax.c,v 1.21.2.5 2002/09/29 18:21:31 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/syntax.c,v 1.21.2.6 2002/11/11 02:42:37 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -94,7 +94,16 @@ syn_insert(
 
 	if ( ssyn->ssyn_oid ) {
 		sir = (struct sindexrec *)
-			ch_calloc( 1, sizeof(struct sindexrec) );
+			SLAP_CALLOC( 1, sizeof(struct sindexrec) );
+		if( sir == NULL ) {
+#ifdef NEW_LOGGING
+			LDAP_LOG( OPERATION, ERR, 
+				"syn_insert: SLAP_CALLOC Error\n", 0, 0, 0 );
+#else
+			Debug( LDAP_DEBUG_ANY, "SLAP_CALLOC Error\n", 0, 0, 0 );
+#endif
+			return LDAP_OTHER;
+		}
 		sir->sir_name = ssyn->ssyn_oid;
 		sir->sir_syn = ssyn;
 		if ( avl_insert( &syn_index, (caddr_t) sir,
@@ -120,7 +129,16 @@ syn_add(
 	Syntax		*ssyn;
 	int		code;
 
-	ssyn = (Syntax *) ch_calloc( 1, sizeof(Syntax) );
+	ssyn = (Syntax *) SLAP_CALLOC( 1, sizeof(Syntax) );
+	if( ssyn == NULL ) {
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, ERR, 
+			"syn_add: SLAP_CALLOC Error\n", 0, 0, 0 );
+#else
+		Debug( LDAP_DEBUG_ANY, "SLAP_CALLOC Error\n", 0, 0, 0 );
+#endif
+		return LDAP_OTHER;
+	}
 
 	AC_MEMCPY( &ssyn->ssyn_syn, syn, sizeof(LDAPSyntax) );
 
@@ -222,7 +240,8 @@ syn_schema_info( Entry *e )
 #endif
 
 #endif
-		attr_merge( e, ad_ldapSyntaxes, vals );
+		if( attr_merge( e, ad_ldapSyntaxes, vals ) )
+			return -1;
 		ldap_memfree( vals[0].bv_val );
 	}
 	return 0;
