@@ -1,5 +1,5 @@
 /* value.c - routines for dealing with values */
-/* $OpenLDAP: pkg/ldap/servers/slapd/value.c,v 1.47.2.5 2002/09/24 00:07:09 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/value.c,v 1.47.2.7 2002/11/26 18:26:19 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -31,15 +31,35 @@ value_add(
 		;	/* NULL */
 
 	if ( *vals == NULL ) {
-		*vals = (BerVarray) ch_malloc( (nn + 1)
+		*vals = (BerVarray) SLAP_MALLOC( (nn + 1)
 		    * sizeof(struct berval) );
+		if( *vals == NULL ) {
+#ifdef NEW_LOGGING
+			 LDAP_LOG( OPERATION, ERR,
+		      "value_add: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#else
+			Debug(LDAP_DEBUG_TRACE,
+		      "value_add: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#endif
+			return LBER_ERROR_MEMORY;
+		}
 		n = 0;
 	} else {
 		for ( n = 0; (*vals)[n].bv_val != NULL; n++ ) {
 			;	/* Empty */
 		}
-		*vals = (BerVarray) ch_realloc( (char *) *vals,
+		*vals = (BerVarray) SLAP_REALLOC( (char *) *vals,
 		    (n + nn + 1) * sizeof(struct berval) );
+		if( *vals == NULL ) {
+#ifdef NEW_LOGGING
+			 LDAP_LOG( OPERATION, ERR,
+		      "value_add: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#else
+			Debug(LDAP_DEBUG_TRACE,
+		      "value_add: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#endif
+			return LBER_ERROR_MEMORY;
+		}
 	}
 
 	v2 = *vals + n;
@@ -63,14 +83,34 @@ value_add_one(
 	BerVarray v2;
 
 	if ( *vals == NULL ) {
-		*vals = (BerVarray) ch_malloc( 2 * sizeof(struct berval) );
+		*vals = (BerVarray) SLAP_MALLOC( 2 * sizeof(struct berval) );
+		if( *vals == NULL ) {
+#ifdef NEW_LOGGING
+			 LDAP_LOG( OPERATION, ERR,
+		      "value_add_one: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#else
+			Debug(LDAP_DEBUG_TRACE,
+		      "value_add_one: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#endif
+			return LBER_ERROR_MEMORY;
+		}
 		n = 0;
 	} else {
 		for ( n = 0; (*vals)[n].bv_val != NULL; n++ ) {
 			;	/* Empty */
 		}
-		*vals = (BerVarray) ch_realloc( (char *) *vals,
+		*vals = (BerVarray) SLAP_REALLOC( (char *) *vals,
 		    (n + 2) * sizeof(struct berval) );
+		if( *vals == NULL ) {
+#ifdef NEW_LOGGING
+			 LDAP_LOG( OPERATION, ERR,
+		      "value_add_one: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#else
+			Debug(LDAP_DEBUG_TRACE,
+		      "value_add_one: SLAP_MALLOC failed.\n", 0, 0, 0 );
+#endif
+			return LBER_ERROR_MEMORY;
+		}
 	}
 
 	v2 = *vals + n;
@@ -284,6 +324,8 @@ value_match(
 	struct berval nv1 = { 0, NULL };
 	struct berval nv2 = { 0, NULL };
 
+	assert( mr != NULL );
+
 	if( !mr->smr_match ) {
 		return LDAP_INAPPROPRIATE_MATCHING;
 	}
